/* rtb_compress.p - compress/decompress binary files
 *
 * Windows Roundtable comes with the freely-available GNU "gzip.exe" utility.
 * Because there are so many platforms that Character Roundtable runs on,
 * Character Roundtable does not ship with the gzip executables.
 *
 * To use gzip compression with Character Roundtable, copy the gzip
 * executable "gzip" (UNIX) or "gzip.exe" (Windows/character) into a gzip
 * directory in the Roundtable program directory, so that
 * SEARCH("gzip/gzip") or SEARCH("gzip/gzip.exe") will locate it.
 *
 * Gzip is recommended for binary file compression because:
 * - it is freely available
 * - it is in common use
 * - it is available (pre-compiled) for most Windows and UNIX platforms
 * - it is available in source form and can be ported to any platform
 *
 * You can modify this program to use other compression methods. However,
 * this is not recommended because:
 * - Roundtable technical support will not provide support for other
 *   compression methods
 * - other compression methods may not be available on as many platforms
 *
 * It is important to use a platform-portable tool, so that you will not have
 * problems if you use Roundtable on a different platform at a later date. For
 * example, UNIX "compress" is not consistent across all UNIX platforms.
 * Also important to keep in mind is that if you use a compression program
 * that uses Windows UI instead of a command-line interface, control might be
 * returned back to your progress session as soon as the Windows program is
 * launched - before the file has been compressed.
 *
 * Parameters:
 * Poperation = "compress":U or "decompress":U
 * Psource = uncompressed file name.
 *    When the (possibly compressed) file is written out from the repository,
 *    it is written to the uncompressed file name rather than to a scratch
 *    file name. So:
 *    - if no compression is used, there's nothing to be done
 *    - in the case of gzip:
 *      - OS-RENAME VALUE(Psource) VALUE(Psource + ".gz":U), and then
 *      - gzip --decompress Psource
 * Ptarget = compressed file name. Used only if Poperation = "compress":U
 * Pdecompress-with = command for decompressing the file. This is stored
 *    in the repository with the object, in case you change compression method
 *    at a later date.
 *    This should be blank if no compression is used.
 *    Do not attempt to use different compression methods for different object
 *    parts. This is not supported by the Roundtable repository.
 *
 * rtb_compress.p is responsible for displaying error messages.
 * RETURN-VALUE should be non-blank if the operation needs to be aborted.
 * Roundtable uses "Cancel":U by convention, but any non-blank value will cause
 * the operation to abort.
 *
 * About the scratch compressed file used here:
 * - when Poperation = "compress":U, Roundtable will delete Ptarget after storing
 * it in the repository.
 * - when Poperation = "decompress":U, this program is responsible for deleting
 * the compressed file after decompressing it. (Gzip works like this anyway.)
 *
 * Disabling Compression:
 *
 *   To disable compression, uncomment the indicated lines in this procedure
 *   that follow the line:
 *
 *          IF Poperation = "compress":U THEN DO:
 */

DEFINE INPUT        PARAMETER Poperation       AS CHARACTER NO-UNDO.
DEFINE INPUT        PARAMETER Psource          AS CHARACTER NO-UNDO.
DEFINE OUTPUT       PARAMETER Ptarget          AS CHARACTER NO-UNDO.
DEFINE INPUT-OUTPUT PARAMETER Pdecompress-with AS CHARACTER NO-UNDO.

DEFINE VARIABLE Malert-title     AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mcommand         AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mcompress-with   AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mdecompress-with AS CHARACTER NO-UNDO.
DEFINE VARIABLE Merror           AS CHARACTER NO-UNDO.
DEFINE VARIABLE Merror-file      AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mgzip-exe        AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mgzip-fix-exe    AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mgzip-rel        AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mgzip-fix-rel    AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mtext            AS CHARACTER NO-UNDO.
DEFINE VARIABLE PTempDir         AS CHARACTER NO-UNDO.

ASSIGN PTempDir = IF OPSYS = "UNIX" THEN
                    SESSION:TEMP-DIRECTORY
                  ELSE
                    DYNAMIC-FUNCTION('fnRtbGetSessionProperty':U,
                                     "temp-directory":U).

IF OPSYS = "UNIX" THEN
  ASSIGN Mgzip-rel     = "gzip/gzip":U      /* relative path to executable */
         Mgzip-fix-rel = "gzip/fixgz":U.
ELSE
  ASSIGN Mgzip-rel     = "gzip/gzip.exe":U
         Mgzip-fix-rel = "gzip\fixgz.exe":U.

ASSIGN Mgzip-exe     = SEARCH(Mgzip-rel)    /* full path to executable     */
       Mgzip-fix-exe = SEARCH(Mgzip-fix-rel)
       Mgzip-exe     = (IF Mgzip-exe <> ? THEN QUOTER(Mgzip-exe) ELSE ?)
       Mgzip-fix-exe = (IF Mgzip-fix-exe <> ? THEN QUOTER(Mgzip-fix-exe) ELSE ?).

ASSIGN Mcompress-with   = "&1 --stdout ~"&2~" > ~"&2.gz~"":U
       Mdecompress-with = "gzip":U + CHR(10) + "&1 --force --decompress &2":U
       Malert-title     = "Roundtable Repository: File Compression".

IF Poperation = "compress":U THEN DO:

  /* ---------------------------------------------------------
     To disable compression, uncomment the following two lines:
     --------------------------------------------------------*/
  /*
   ASSIGN Pdecompress-with = "":U.
   RETURN.
  */

  /*
   Do not attempt to compress already compressed (gzip) files
  */
  IF pSource MATCHES "*~.tgz" OR
     pSource matches "*~.gz"  OR
     pSource matches "*~.gzip" THEN DO:
    Pdecompress-with = "".
    RETURN.
  END.

  IF Mgzip-exe = ? THEN DO:
    Merror = SUBSTITUTE(
 "'&1' was not found. Add '&1' to the Roundtable program directory to compress~
 binary files using gzip. Modify rtb/p/rtb_compress.p to eliminate this message~
 or to change file compression methods.",Mgzip-rel).
    IF OPSYS = "UNIX":U THEN DO:
      MESSAGE Merror VIEW-AS ALERT-BOX ERROR TITLE Malert-title.
      RETURN "Cancel":U.
    END.
    ELSE
      RETURN Merror.
  END.

  ASSIGN Mcommand         = SUBSTITUTE(Mcompress-with,Mgzip-exe, Psource)
         Pdecompress-with = Mdecompress-with.

  IF OPSYS = "UNIX":U THEN
    OS-COMMAND SILENT VALUE(Mcommand).
  ELSE DO:
    Mcommand = REPLACE(Mcommand,"~%","~%%"). /* A .bat file interprets % as parameters, */
    OUTPUT TO VALUE(PTempDir + "TEMP.BAT").  /* so we have to replace them with %%. */
    PUT UNFORMATTED "@ECHO OFF" SKIP.
    PUT UNFORMATTED Mcommand.
    OUTPUT CLOSE.
    OS-COMMAND SILENT VALUE(PTempDir + "TEMP.BAT").
  END. /* else do (if opsys) */

  /*
   Test the generated archive...
  */
  ASSIGN 
    Mcommand = SUBSTITUTE("&1 --test &2":U,Mgzip-exe,QUOTER(Psource + ".gz":U))
    Merror-file = Psource + ".err":U.

  /* Remove any existing error file */
  OS-DELETE VALUE(Merror-file) NO-ERROR. 

  /* Run gzip test */
  OUTPUT TO VALUE(Merror-file).
  OS-COMMAND SILENT VALUE(Mcommand).
  OUTPUT CLOSE.

  /* Read the test results */
  INPUT FROM VALUE(Merror-file) NO-ECHO.
  REPEAT:
    IMPORT UNFORMATTED Mtext.
    Merror = Merror + (IF Merror = "":U THEN "":U ELSE CHR(10)) + Mtext.
  END.
  INPUT CLOSE.

  /* Done with error file - remove it */
  OS-DELETE VALUE(Merror-file).

  /* If test results not blank, then something went wrong! */
  IF Merror <> "":U THEN DO:
    Merror = SUBSTITUTE("&1&3&3Gzip compress failed.&3&3&2&3&3See rtb/p/rtb_compress to review the compression method.",
                        Psource,Merror,CHR(10)).
    IF OPSYS = "UNIX":U THEN DO:
      MESSAGE Merror VIEW-AS ALERT-BOX ERROR TITLE Malert-title.
      RETURN "Cancel":U.    
    END.
    ELSE
      RETURN Merror.
  END.

  ASSIGN Ptarget = SEARCH(Psource + ".gz":U).

  RETURN. /* necessary for clearing RETURN-VALUE */
END. /* compress */

IF Poperation = "decompress":U THEN DO:

  /* --- Check for the compression method used at check-in time ---
   * As shipped, rtb_compress.p only supports "gzip".
   * -------------------------------------------------------------- */
  IF ENTRY(1,Mdecompress-with,CHR(10)) = "gzip":U THEN
     ASSIGN Mdecompress-with = ENTRY(2,Mdecompress-with,CHR(10)).
  ELSE DO:
    Merror = SUBSTITUTE("Unknown compression method:&2'&1'&2.Cannot~
 decompress extracted file.&2See rtb/p/rtb_compress.p to~
 review your compression method.",
                        ENTRY(1,Mdecompress-with,CHR(10)),
                        CHR(10)).
    IF OPSYS = "UNIX":U THEN DO:
      MESSAGE Merror VIEW-AS ALERT-BOX ERROR TITLE Malert-title.
      RETURN "Cancel":U.
    END.
    ELSE
      RETURN Merror.
  END.

  /* --- Check that the gzip executable exists --- */
  IF Mgzip-exe = ? THEN DO:
    Merror = SUBSTITUTE("'&1' was not found.&2Cannot decompress extracted file.&2~
See rtb/p/rtb_compress.p to review your compression method.",
                        Mgzip-exe,CHR(10)).
     IF OPSYS = "UNIX":U THEN DO:
       MESSAGE Merror VIEW-AS ALERT-BOX ERROR TITLE Malert-title.
       RETURN "Cancel":U.
     END.
     ELSE
       RETURN Merror.
  END.

  /* --- Move the file from Psource to Psource.gz ---
   * We have to do this because the file was extracted to VALUE(Psource).
   * Note that gzip --decompress will decompress "Psource.gz" to "Psource".
   * Also note that stupid OS-RENAME will CAPITALIZE the target
   * filename (at least it does on NT). Grumble. Use mv or move instead.
   * ------------------------------------------------ */
  IF OPSYS = "WIN32":U THEN
    ASSIGN Psource = REPLACE(Psource,"/","~\").

  OS-DELETE VALUE("~"" + Psource + ".gz" + "~"":U).

  IF OPSYS = "UNIX":U THEN
     OS-COMMAND SILENT VALUE("mv " + "~"" + Psource + "~"" 
                             + " " + "~"" + Psource + ".gz" + "~"":U).
  ELSE DO:
    Mcommand = "move " 
             + "~"" + Psource + "~"" + " " 
             + "~"" + Psource + ".gz" + "~"":U.
    Mcommand = REPLACE(Mcommand,"~%","~%%").
    OUTPUT TO VALUE(PTempDir + "TEMP.BAT").
    PUT UNFORMATTED "@ECHO OFF" SKIP.
    PUT UNFORMATTED Mcommand.
    OUTPUT CLOSE.
    OS-COMMAND SILENT VALUE(PTempDir + "TEMP.BAT").
  END.


  /* --- Now execute gzip to decompress the file --- */
  ASSIGN
    Mcommand = SUBSTITUTE(ENTRY(2,Pdecompress-with,CHR(10)),Mgzip-exe,"~"" 
                          + Psource + "~"")
    Merror-file = Psource + ".err":U.

  IF OPSYS = "UNIX":U THEN DO:
    OUTPUT TO VALUE(Merror-file).
    OS-COMMAND SILENT VALUE(Mcommand).
    OUTPUT CLOSE.
  END.
  ELSE DO:
    Mcommand = REPLACE(Mcommand,"~%","~%%").
    OUTPUT TO VALUE(PTempDir + "TEMP.BAT").
    PUT UNFORMATTED "@ECHO OFF" SKIP.
    PUT UNFORMATTED Mcommand.
    OUTPUT CLOSE.
    OUTPUT TO VALUE(Merror-file).
    OS-COMMAND SILENT VALUE(PTempDir + "TEMP.BAT").
    OUTPUT CLOSE.

    /* The .gz file should be gone now */
    IF SEARCH(Psource + ".gz") <> ? THEN DO:
      IF SEARCH(Mgzip-fix-exe) <> ? THEN DO:
        OUTPUT TO VALUE(PTempDir + "FIXGZIP.BAT").
        PUT UNFORMATTED "@ECHO OFF" SKIP.
        PUT UNFORMATTED Mgzip-fix-exe + " ~"" + Psource + ".gz~" ~"" + PTempDir + "fixgzip.gz~"" SKIP.
        PUT UNFORMATTED "move ~"" + PTempDir + "fixgzip.gz~" " + "~"" + Psource + ".gz" + "~"" SKIP.
        OUTPUT CLOSE.
        OS-COMMAND SILENT VALUE(PTempDir + "FIXGZIP.BAT").
        OS-DELETE VALUE(PTempDir + "FIXGZIP.BAT").
        OUTPUT TO VALUE(Merror-file).
        OS-COMMAND SILENT VALUE(PTempDir + "TEMP.BAT").
        OUTPUT CLOSE.
        OS-DELETE VALUE(PTempDir + "TEMP.BAT").
      END. /* IF SEARCH(Psource + ".gz") <> ? THEN */
    END. /* try-zipfix */
  END. /* gui-decomp */

  IF SEARCH(Psource) = ? THEN DO:
    /* Read the command output */
    INPUT FROM VALUE(Merror-file) NO-ECHO.
    REPEAT:
      IMPORT UNFORMATTED Mtext.
      Merror = Merror + (IF Merror = "":U THEN "":U ELSE CHR(10)) + Mtext.
    END.
    INPUT CLOSE.

    /* Done with error file - remove it */
    OS-DELETE VALUE(Merror-file).

    Merror = SUBSTITUTE("Decompress for '&1' failed.&3&3&4&3&3&2&3&3~
See rtb/p/rtb_compress to review the compression method.",
                       Psource,Mcommand,CHR(10),Merror).
    IF OPSYS = "UNIX":U THEN DO:
      MESSAGE Merror VIEW-AS ALERT-BOX ERROR TITLE Malert-title.
      RETURN "Cancel":U.
    END.
    ELSE
      RETURN Merror.
  END.

  /* If we got this far, we need to remove the error file  */
  OS-DELETE VALUE(Merror-file).

  RETURN. /* necessary for clearing RETURN-VALUE */

END. /* decompress */
